/*
 * Implementacija liste pomocu statickog niza.
 */
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

// Maximalna duzina liste (kapacitet)
#define MAX_LIST_SIZE 50

// Pozicija prvog elementa liste. Indeksiranje u C-u krece od 0, stoga je to i
// vrednost pozicije.
#define FIRST_ELEMENT_LIST_POSITION 0

typedef int element_type;

typedef int position;

// Definicija strukture kojom se opisuje lista. Lista se opisuje nizom svojih
// elemenata, kao i vrednoscu koja predstavlja indeks poslednjeg elementa liste, 
// i koji se menja kako se lista povecava/smanjuje.
typedef struct _list {
        position last_position;
        element_type elements[MAX_LIST_SIZE];
} list;

/*
 * Izracunava poziciju za jednu iza poslednjeg elementa liste.
 */
position end(list L) {
         return L.last_position + 1;
}

/*
 * Izracunava poziciju prvog elementa liste.
 */
position first(list L) {
         return FIRST_ELEMENT_LIST_POSITION;
}

/*
 * Vraca poziciju narednog elementa liste u odnosu na datu poziciju. F-ja proverava
 * ispravnost zadate pozicije koja mora biti u intervalu [0, end(L)), i prekida
 * izvrsavanje ukoliko to nije slucaj.
 */
position next(position i, list L) {
         assert(i >= FIRST_ELEMENT_LIST_POSITION && i < end(L));
         return i + 1;
}

/*
 * Vraca poziciju prethodnog elementa liste u odnosu na datu poziciju. F-ja proverava
 * ispravnost zadate pozicije koja mora biti u intervalu (0, end(L)], i prekida
 * izvrsavanje ukoliko to nije slucaj.
 */
position previous(position i, list L) {
         assert(i > FIRST_ELEMENT_LIST_POSITION && i <= end(L));
         return i - 1;
}

/*
 * Prazni listu.
 */
void makeNull(list *L) {
     L->last_position = FIRST_ELEMENT_LIST_POSITION - 1;
}

void initList(list *L) {
     L->last_position = FIRST_ELEMENT_LIST_POSITION - 1;
}

/*
 * Vraca element liste na datoj poziciji. F-ja proverava ispravnost zadate pozicije
 * koja mora biti u granicama dimenzije liste ([0, end(L)), i prekida izvrasavanje
 * ako to nije slucaj.
 */
element_type retrieve(position i, list L) {
         assert(i >= FIRST_ELEMENT_LIST_POSITION && i < end(L));
         return L.elements[i];
}

/*
 * Pronalazi poziciju prvog pojavljivanja elementa x u listi L. Ukoliko x ne pripada
 * listi kao rezultat se vraca end(L).
 */
position locate(element_type x, list L) {
         position i;
         for(i = FIRST_ELEMENT_LIST_POSITION; i < end(L); i++) {
               if(retrieve(i, L) == x) {
                              return i;
               }
         }
         
         return end(L);
}

/*
 * Umece element x na poziciju p u listu L. F-ja proverava da li lista ima dovoljno
 * mesta za smestanje zadatog elementa. Takodje proverava ispravnost pozicije, koja
 * mora biti u granicama duzine liste. U oba slucaja se prekida izvrsavanje
 * ukoliko to nije slucaj.
 */
void insert(element_type x, position p, list *L) {
     assert(end(*L) < MAX_LIST_SIZE);
     assert(p >= FIRST_ELEMENT_LIST_POSITION && p <= end(*L));
     
     position i;
     // Ciklicki se pomeraju udesno elementi liste, pocev od pozicije p.
     for(i = L->last_position; i >= p; i--) {
           L->elements[i + 1] = L->elements[i];
     }
     
     // Umece se dati element na zadatu poziciju, i povecava se stvarna duzina liste
     // za 1.
     L->elements[p] = x;
     
     L->last_position++;
}

/*
 * Brise element iz liste L na poziciji p. F-ja proverava ispravnost zadate
 * pozicije. 
 */
void delete(position p, list *L) {
     assert(p >= FIRST_ELEMENT_LIST_POSITION && p < end(*L));
     
     position i;
     for(i = p; i < L->last_position; i++) {
           L->elements[i] = L->elements[i + 1];
     }
     
     L->last_position--;
}

/*
 * Ispisuje elemente liste u obliku (a1, a2, ..., an).
 */
void print(list L) {
     int i;
     
     printf("(");
     // Ispisuje sve elemente liste do pretposlednjeg, da se ne na izlazu
     // ne bi dobio zarez viska iza poslednjeg elementa.
     for(i = FIRST_ELEMENT_LIST_POSITION; i < L.last_position; i++) {
           printf("%d, ",  retrieve(i, L));
     }
     // Ispisuje se poslednji element liste odvojeno.
     if(L.last_position >= FIRST_ELEMENT_LIST_POSITION) {
           printf("%d",  retrieve(L.last_position, L));
     }
     printf(")\n");
}

int main(int argc, char *argv[])
{
    list L;
    initList(&L); // Vazan korak inicijalizacije liste i postavljanje 
                   // indeksa poslednjeg elementa liste na -1.
    
    // Lista je inicijalno prazna.
    print(L);
    
    // Dodaju se tri elementa na prvu poziciju.
    for(int i = 0; i < 3; i++) {
            insert(i, 0, &L);
    } 
    
    // Sad je lista (2, 1, 0).
    print(L);
    
    // Brise se element sa druge pozicije.
    delete(1, &L);
    
    // Sada je lista (2, 0).
    print(L);
    
    system("PAUSE");
    return 0;
}
